/* Fast userspace semaphores. 
 * Based on work by Matthew Kirkwood <matthew@hairy.beasts.org>. */
#include "usersem.h"
#include <stdint.h>
#include <sys/stat.h>
#include <fcntl.h>
#define __USE_UNIX98
#include <unistd.h>
#include <stdio.h>

#include ARCH_ATOMICS_C

int usem_open(void)
{
	return open("/dev/usem", O_RDWR);
}

void usem_close(int usem_fd)
{
	close(usem_fd);
}

void usem_init(struct usersem *sem)
{
	sem->count = 1;
	sem->num_sleepers = 0;
	__commit();
}

/* Down and up operations */
int usem_down(struct usersem *sem, int usem_fd)
{
	if (__down(&sem->count)) {
		/* Slow path */
		char dummy;
		return pread(usem_fd, &dummy, 1, (off_t)sem);
	}
	return 0;
}

int usem_up(struct usersem *sem, int usem_fd)
{
	if (__up(&sem->count)) {
		/* Slow path */
		return pwrite(usem_fd, "", 1, (off_t)sem);
	}
	return 0;
}
